@@ -17,6 +17,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 #include <errno.h>
+#include <linux/net_tstamp.h>
 #include <poll.h>
 #include <stdlib.h>
 #include <string.h>
@@ -73,6 +74,7 @@ struct clock_subscriber {
 };
 
 struct clock {
+	enum clock_type type;
 	struct config *config;
 	clockid_t clkid;
 	struct servo *servo;
@@ -798,19 +800,19 @@ static void clock_remove_port(struct clock *c, struct port *p)
 	port_close(p);
 }
 
-struct clock *clock_create(struct config *config, int phc_index,
-			   struct interfaces_head *ifaces,
-			   struct default_ds *dds)
+struct clock *clock_create(enum clock_type type, struct config *config,
+			   const char *phc_device)
 {
 	enum timestamp_type timestamping =
 		config_get_int(config, NULL, "time_stamping");
 	int fadj = 0, max_adj = 0, sw_ts = timestamping == TS_SOFTWARE ? 1 : 0;
 	enum servo_type servo = config_get_int(config, NULL, "clock_servo");
+	int phc_index, required_modes = 0;
 	struct clock *c = &the_clock;
 	struct port *p;
-	char phc[32];
-	struct interface *udsif = &c->uds_interface;
-	struct interface *iface;
+	unsigned char oui[OUI_LEN];
+	char phc[32], *tmp;
+	struct interface *iface, *udsif = &c->uds_interface;
 	struct timespec ts;
 	int sfl;
 
@@ -820,8 +822,154 @@ struct clock *clock_create(struct config *config, int phc_index,
 	if (c->nports)
 		clock_destroy(c);
 
+	switch (type) {
+	case CLOCK_TYPE_ORDINARY:
+	case CLOCK_TYPE_BOUNDARY:
+		c->type = type;
+		break;
+	case CLOCK_TYPE_P2P:
+	case CLOCK_TYPE_E2E:
+	case CLOCK_TYPE_MANAGEMENT:
+		return NULL;
+	}
+
+	/* Initialize the defaultDS. */
+	c->dds.clockQuality.clockClass =
+		config_get_int(config, NULL, "clockClass");
+	c->dds.clockQuality.clockAccuracy =
+		config_get_int(config, NULL, "clockAccuracy");
+	c->dds.clockQuality.offsetScaledLogVariance =
+		config_get_int(config, NULL, "offsetScaledLogVariance");
+
+	c->desc.productDescription.max_symbols = 64;
+	c->desc.revisionData.max_symbols = 32;
+	c->desc.userDescription.max_symbols = 128;
+
+	tmp = config_get_string(config, NULL, "productDescription");
+	if (count_char(tmp, ';') != 2 ||
+	    static_ptp_text_set(&c->desc.productDescription, tmp)) {
+		pr_err("invalid productDescription '%s'", tmp);
+		return NULL;
+	}
+	tmp = config_get_string(config, NULL, "revisionData");
+	if (count_char(tmp, ';') != 2 ||
+	    static_ptp_text_set(&c->desc.revisionData, tmp)) {
+		pr_err("invalid revisionData '%s'", tmp);
+		return NULL;
+	}
+	tmp = config_get_string(config, NULL, "userDescription");
+	if (static_ptp_text_set(&c->desc.userDescription, tmp)) {
+		pr_err("invalid userDescription '%s'", tmp);
+		return NULL;
+	}
+	tmp = config_get_string(config, NULL, "manufacturerIdentity");
+	if (OUI_LEN != sscanf(tmp, "%hhx:%hhx:%hhx", &oui[0], &oui[1], &oui[2])) {
+		pr_err("invalid manufacturerIdentity '%s'", tmp);
+		return NULL;
+	}
+	memcpy(c->desc.manufacturerIdentity, oui, OUI_LEN);
+
+	c->dds.domainNumber = config_get_int(config, NULL, "domainNumber");
+
+	if (config_get_int(config, NULL, "slaveOnly")) {
+		c->dds.flags |= DDS_SLAVE_ONLY;
+	}
+	if (config_get_int(config, NULL, "twoStepFlag")) {
+		c->dds.flags |= DDS_TWO_STEP_FLAG;
+	}
+	c->dds.priority1 = config_get_int(config, NULL, "priority1");
+	c->dds.priority2 = config_get_int(config, NULL, "priority2");
+
+	if (!config_get_int(config, NULL, "gmCapable") &&
+	    c->dds.flags & DDS_SLAVE_ONLY) {
+		pr_err("Cannot mix 1588 slaveOnly with 802.1AS !gmCapable");
+		return NULL;
+	}
+	if (!config_get_int(config, NULL, "gmCapable") ||
+	    c->dds.flags & DDS_SLAVE_ONLY) {
+		c->dds.clockQuality.clockClass = 255;
+	}
+
+	if (!(c->dds.flags & DDS_TWO_STEP_FLAG)) {
+		switch (config_get_int(config, NULL, "time_stamping")) {
+		case TS_SOFTWARE:
+		case TS_LEGACY_HW:
+			pr_err("one step is only possible "
+			       "with hardware time stamping");
+			return NULL;
+		case TS_HARDWARE:
+			if (config_set_int(config, "time_stamping", TS_ONESTEP))
+				return NULL;
+			break;
+		case TS_ONESTEP:
+			break;
+		}
+	}
+
+	/* Check the time stamping mode on each interface. */
+	switch (config_get_int(config, NULL, "time_stamping")) {
+	case TS_SOFTWARE:
+		required_modes |= SOF_TIMESTAMPING_TX_SOFTWARE |
+			SOF_TIMESTAMPING_RX_SOFTWARE |
+			SOF_TIMESTAMPING_SOFTWARE;
+		break;
+	case TS_LEGACY_HW:
+		required_modes |= SOF_TIMESTAMPING_TX_HARDWARE |
+			SOF_TIMESTAMPING_RX_HARDWARE |
+			SOF_TIMESTAMPING_SYS_HARDWARE;
+		break;
+	case TS_HARDWARE:
+	case TS_ONESTEP:
+		required_modes |= SOF_TIMESTAMPING_TX_HARDWARE |
+			SOF_TIMESTAMPING_RX_HARDWARE |
+			SOF_TIMESTAMPING_RAW_HARDWARE;
+		break;
+	}
+	STAILQ_FOREACH(iface, &config->interfaces, list) {
+		if (iface->ts_info.valid &&
+		    ((iface->ts_info.so_timestamping & required_modes) != required_modes)) {
+			pr_err("interface '%s' does not support "
+			       "requested timestamping mode", iface->name);
+			return NULL;
+		}
+	}
+
+	iface = STAILQ_FIRST(&config->interfaces);
+
+	/* determine PHC Clock index */
+	if (config_get_int(config, NULL, "free_running")) {
+		phc_index = -1;
+	} else if (config_get_int(config, NULL, "time_stamping") == TS_SOFTWARE ||
+		   config_get_int(config, NULL, "time_stamping") == TS_LEGACY_HW) {
+		phc_index = -1;
+	} else if (phc_device) {
+		if (1 != sscanf(phc_device, "/dev/ptp%d", &phc_index)) {
+			pr_err("bad ptp device string");
+			return NULL;
+		}
+	} else if (iface->ts_info.valid) {
+		phc_index = iface->ts_info.phc_index;
+	} else {
+		pr_err("PTP device not specified and automatic determination"
+		       " is not supported. Please specify PTP device.");
+		return NULL;
+	}
+	if (phc_index >= 0) {
+		pr_info("selected /dev/ptp%d as PTP clock", phc_index);
+	}
+
+	if (generate_clock_identity(&c->dds.clockIdentity, iface->name)) {
+		pr_err("failed to generate a clock identity");
+		return NULL;
+	}
+
+	/* Configure the UDS. */
 	snprintf(udsif->name, sizeof(udsif->name), "%s",
 		 config_get_string(config, NULL, "uds_address"));
+	if (config_set_section_int(config, udsif->name,
+				   "announceReceiptTimeout", 0)) {
+		return NULL;
+	}
 	if (config_set_section_int(config, udsif->name,
 				    "delay_mechanism", DM_AUTO)) {
 		return NULL;
@@ -841,7 +989,6 @@ struct clock *clock_create(struct config *config, int phc_index,
 	c->kernel_leap = config_get_int(config, NULL, "kernel_leap");
 	c->utc_offset = CURRENT_UTC_OFFSET;
 	c->time_source = config_get_int(config, NULL, "timeSource");
-	c->desc = dds->clock_desc;
 
 	if (c->free_running) {
 		c->clkid = CLOCK_INVALID;
@@ -911,8 +1058,6 @@ struct clock *clock_create(struct config *config, int phc_index,
 		}
 	}
 
-	c->dds = dds->dds;
-
 	/* Initialize the parentDS. */
 	clock_update_grandmaster(c);
 	c->dad.pds.parentStats                           = 0;
@@ -941,7 +1086,7 @@ struct clock *clock_create(struct config *config, int phc_index,
 	clock_fda_changed(c);
 
 	/* Create the ports. */
-	STAILQ_FOREACH(iface, ifaces, list) {
+	STAILQ_FOREACH(iface, &config->interfaces, list) {
 		if (clock_add_port(c, phc_index, timestamping, iface)) {
 			pr_err("failed to open port %s", iface->name);
 			return NULL;
@@ -991,6 +1136,11 @@ UInteger8 clock_domain_number(struct clock *c)
 	return c->dds.domainNumber;
 }
 
+struct port *clock_first_port(struct clock *c)
+{
+	return LIST_FIRST(&c->ports);
+}
+
 void clock_follow_up_info(struct clock *c, struct follow_up_info_tlv *f)
 {
 	c->status.cumulativeScaledRateOffset = f->cumulativeScaledRateOffset;
@@ -1553,9 +1703,9 @@ struct clock_description *clock_description(struct clock *c)
 	return &c->desc;
 }
 
-int clock_num_ports(struct clock *c)
+enum clock_type clock_type(struct clock *c)
 {
-	return c->nports;
+	return c->type;
 }
 
 void clock_check_ts(struct clock *c, struct timespec ts)
