@@ -103,6 +103,7 @@ struct port {
 	TimeInterval        peerMeanPathDelay;
 	Integer8            logAnnounceInterval;
 	UInteger8           announceReceiptTimeout;
+	int                 announce_span;
 	UInteger8           syncReceiptTimeout;
 	UInteger8           transportSpecific;
 	Integer8            logSyncInterval;
@@ -376,6 +377,7 @@ static int add_foreign_master(struct port *p, struct ptp_message *m)
 			return 0;
 		}
 		memset(fc, 0, sizeof(*fc));
+		TAILQ_INIT(&fc->messages);
 		LIST_INSERT_HEAD(&p->foreign_masters, fc, list);
 		fc->port = p;
 		fc->dataset.sender = m->header.sourcePortIdentity;
@@ -705,12 +707,7 @@ static int port_management_fill_response(struct port *target,
 		buf = tlv->data;
 		cd->clockType = (UInteger16 *) buf;
 		buf += sizeof(*cd->clockType);
-		if (clock_num_ports(target->clock) > 1) {
-			*cd->clockType = CLOCK_TYPE_BOUNDARY;
-		} else {
-			*cd->clockType = CLOCK_TYPE_ORDINARY;
-		}
-
+		*cd->clockType = clock_type(target->clock);
 		cd->physicalLayerProtocol = (struct PTPText *) buf;
 		switch(transport_type(target->trp)) {
 		case TRANS_UDP_IPV4:
@@ -962,7 +959,7 @@ static int port_set_announce_tmo(struct port *p)
 {
 	return set_tmo_random(p->fda.fd[FD_ANNOUNCE_TIMER],
 			      p->announceReceiptTimeout,
-			      ANNOUNCE_SPAN, p->logAnnounceInterval);
+			      p->announce_span, p->logAnnounceInterval);
 }
 
 static int port_set_delay_tmo(struct port *p)
@@ -2569,6 +2566,7 @@ struct port *port_open(int phc_index,
 	p->name = interface->name;
 	p->asymmetry = config_get_int(cfg, p->name, "delayAsymmetry");
 	p->asymmetry <<= 16;
+	p->announce_span = transport == TRANS_UDS ? 0 : ANNOUNCE_SPAN;
 	p->follow_up_info = config_get_int(cfg, p->name, "follow_up_info");
 	p->freq_est_interval = config_get_int(cfg, p->name, "freq_est_interval");
 	p->hybrid_e2e = config_get_int(cfg, p->name, "hybrid_e2e");
